# This is from http://www.gramps-project.org/wiki/index.php?title=Geographic_Report with
# changes from Joachim Breitner <mail@joachim-breitner.de>
#
# Gramps - a GTK+/GNOME based genealogy program
#
# Copyright (C) 2000-2006  Donald N. Allingham
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#



"""
Contains the interface to allow to write place data: longitude latitude. 
One backend for writing is supported: the KML file format used by google earth

info: bm at cage dot UGent dot be
"""

#-------------------------------------------------------------------------
#
# load standard python libraries
#
#-------------------------------------------------------------------------
import time
import os
import codecs
from gettext import gettext as _
import Utils

from Calendar import NewReportOptions, FilterWidget, CheckWidget
from ReportBase._CommandLineReport import CommandLineReport

#-------------------------------------------------------------------------
#
# load GRAMPS libraries
#
#-------------------------------------------------------------------------
import RelLib

#------------------------------------------------------------------------
#
# Set up logging
#
#------------------------------------------------------------------------
import logging
log = logging.getLogger(".WriteDescendNet")


#-------------------------------------------------------------------------
#
#   backend: KML writer for the actual writing of geographic data
#
#-------------------------------------------------------------------------
class KmlWriter:
    """
    Writes output to a KML file.
    """
    def __init__(self, title, placelines={}, placestyles={}):
        """
        Initializes, but does not write, a KML file.
           title : title of the KML group
           placestyle: Different styles to show data in kml. A dictionary with 
                       key stylename, data style
           placemarks: A dictionary with the places that must be shown, 
                    the index is the placehandle, the data with the index is
                    [placename, lat, lon, events], where events is a list with
                    every element a list [(personname/familyname, 
                        personname/familyname_short, type = F,M,U,Fam), 
                        eventname, eventdescription,
                        eventRole, eventDate, RegularDateTrueFalse], 
                    where RegularDateTrueFalse is True if eventDate is a 
                    regular date. The type is F = Female, M = Male, U = unknown
                    for personal events, and Fam for family events
        """
        self.title = title
        self.styles = placestyles
        self.families = placelines
        
    def write(self,filename):
        """
        Write the data to the specified file.
        """
        base = os.path.dirname(filename)
        if os.path.isdir(base):
            if not os.access(base,os.W_OK) or not os.access(base,os.R_OK):
                ErrorDialog(_('Failure writing %s') % filename,
                            _("The file cannot be saved because you do "
                              "not have permission to write to the directory. "
                              "Please make sure you have write access to the "
                              "directory and try again."))
                return 0
            
        if os.path.exists(filename):
            if not os.access(filename,os.W_OK):
                ErrorDialog(_('Failure writing %s') % filename,
                            _("The file cannot be saved because you do "
                              "not have permission to write to the file. "
                              "Please make sure you have write access to the "
                              "file and try again."))
                return 0
        
        self.fileroot = os.path.dirname(filename)

        try:
            g = open(filename,"w")
        except IOError,msg:
            ErrorDialog(_('Failure writing %s') % filename,msg)
            return 0
        self.g = codecs.getwriter("utf8")(g)

        self.write_kml_data()
        g.close()
        return 1

    def write_handle(self,handle):
        """
        Write the data to the specified file handle.
        """
        g = handle

        self.g = codecs.getwriter("utf8")(g)

        self.write_kml_data()
        g.close()
        return 1
            
    def write_kml_data(self):
        self.write_kml_header()

        # set all placemarks
        for gen, families in self.families.items():
            self.write_gen(gen, families) 
        self.g.write("</Document>\n")
        self.g.write("</kml> \n")

    def write_gen(self, gen, families):

        self.g.write('''
        <Folder>
            <name>Generation %d</name>
            <styleUrl>#folderStyleGen%d</styleUrl>
        ''' % (gen,gen))
        if gen>1:
            self.g.write('''<open>0</open>''')
            self.g.write('''<visibility>0</visibility>''')
        else:
            self.g.write('''<open>1</open>''')
            self.g.write('''<visibility>1</visibility>''')
        for family in families:
            self.write_family(family)

        self.g.write('</Folder>')


    def write_kml_header(self):
        self.g.write('<?xml version="1.0" encoding="UTF-8"?>\n')
        self.g.write('<kml xmlns="http://earth.google.com/kml/2.1">\n')
        # some info
        date = time.localtime(time.time())
        self.g.write('<!-- created date="%04d-%02d-%02d\" -->\n'%
                     (date[0],date[1],date[2]) )

        # start the document
        self.g.write("<Document>\n")
        # set the name of the document
        self.g.write("<name>%s</name>\n" %(self.title))
        self.g.write("<open>1</open>\n")
        self.g.write("<visibility>0</visibility>\n")
        # set the default styles used
        self.g.write("""
          <Style id="lineStyleGen1">
            <LineStyle>
              <color>a0ff0000</color>
              <colorMode>normal</colorMode>
              <width>7</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen1">
            <ListStyle>
              <bgColor>a0ff0000</bgColor>
           </ListStyle>
          </Style>
          <Style id="lineStyleGen2">
            <LineStyle>
              <color>a0ffbf00</color>
              <colorMode>normal</colorMode>
              <width>6</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen2">
            <ListStyle>
              <bgColor>a0ffbf00</bgColor>
           </ListStyle>
          </Style>
          <Style id="lineStyleGen3">
            <LineStyle>
              <color>a080ff00</color>
              <colorMode>normal</colorMode>
              <width>5</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen3">
            <ListStyle>
              <bgColor>a080ff00</bgColor>
           </ListStyle>
          </Style>
          <Style id="lineStyleGen4">
            <LineStyle>
              <color>a000ff40</color>
              <colorMode>normal</colorMode>
              <width>4</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen4">
            <ListStyle>
              <bgColor>a000ff40</bgColor>
           </ListStyle>
          </Style>
          <Style id="lineStyleGen5">
            <LineStyle>
              <color>a000ffff</color>
              <colorMode>normal</colorMode>
              <width>3</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen5">
            <ListStyle>
              <bgColor>a000ffff</bgColor>
           </ListStyle>
          </Style>
          <Style id="lineStyleGen6">
            <LineStyle>
              <color>a00040ff</color>
              <colorMode>normal</colorMode>
              <width>2</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen6">
            <ListStyle>
              <bgColor>a00040ff</bgColor>
           </ListStyle>
          </Style>
          <Style id="lineStyleGen7">
            <LineStyle>
              <color>a08000ff</color>
              <colorMode>normal</colorMode>
              <width>1</width>
           </LineStyle>
          </Style>
          <Style id="folderStyleGen7">
            <ListStyle>
              <bgColor>a08000ff</bgColor>
              <colorMode>normal</colorMode>
              <width>1</width>
           </ListStyle>
          </Style>
          <Style id="lineStylePartner">
            <LineStyle>
              <color>a0000000</color>
              <colorMode>normal</colorMode>
              <width>1</width>
           </LineStyle>
          </Style>
       """)
        self.g.write("""
          <Style id="grampsM"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventM.png")) )
        self.g.write("""
          <Style id="grampsF"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventF.png")) )
        self.g.write("""
          <Style id="grampsU"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventU.png")) )
        self.g.write("""
          <Style id="grampsFam"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFam.png")) )
        self.g.write("""
          <Style id="grampsFammar"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFammar.png")) )
        #icons events female
        self.g.write("""
          <Style id="grampsFbapt"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFbapt.png")) )
        self.g.write("""
          <Style id="grampsFbirth"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFbirth.png")) )
        self.g.write("""
          <Style id="grampsFdeath"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFdeath.png")) )
        self.g.write("""
          <Style id="grampsFburial"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventFburial.png")) )
        #icons male
        self.g.write("""
          <Style id="grampsMbapt"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMbapt.png")) )
        self.g.write("""
          <Style id="grampsMbirth"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMbirth.png")) )
        self.g.write("""
          <Style id="grampsMdeath"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMdeath.png")) )
        self.g.write("""
          <Style id="grampsMburial"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventMburial.png")) )
        #icons unknown
        self.g.write("""
          <Style id="grampsUbapt"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUbapt.png")) )
        self.g.write("""
          <Style id="grampsUbirth"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUbirth.png")) )
        self.g.write("""
          <Style id="grampsUdeath"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUdeath.png")) )
        self.g.write("""
          <Style id="grampsUburial"> 
            <BalloonStyle>
              <!-- a background color for the balloon -->
              <color>00ffffff</color>
              <!-- styling of the balloon text -->
              <text><![CDATA[
              <b><font color="#CC0000" size="+3">$[name]</font></b>
              <br/><br/>
              $[description]
              <br/>
              <br/>
              <!-- insert the to/from hyperlinks -->
              $[geDirections]
              ]]></text>
            </BalloonStyle>
            <IconStyle id="grampsIcon">
              <color>ffffffff</color>
              <colorMode>normal</colorMode>
              <scale>1</scale>
              <Icon>%s</Icon> 
              <hotSpot x="0.5"  y="0.5" 
                  xunits="fraction" yunits="fraction"/>    <!-- kml:vec2Type -->
            </IconStyle>
          </Style>
        """ % (os.path.join(const.image_dir, "KML","eventUburial.png")) )

    def write_family(self, family):
        if family['lines']:
            self.g.write("""
            <Placemark>
                <styleUrl>#lineStyleGen%(gen)d</styleUrl>
                <name>%(name)s</name>
                <description>%(description)s</description>
                <Snippet>%(snippet)s</Snippet>
                <LookAt>
                  <longitude>%(fromlon).14f</longitude>
                  <latitude>%(fromlat).14f</latitude>
                  <range>50000</range>
                </LookAt>
                <MultiGeometry>
            """ % family )

            for line in family['lines']:
                self.write_line(line)

            self.g.write("""
                </MultiGeometry>
            </Placemark>
            """)
        if family['inlawlines']:
            self.g.write("""
            <Placemark>
                <styleUrl>#lineStylePartner</styleUrl>
                <name>%(name)s - Schwiegers</name> <!--
                <description>%(description)s</description>
                <Snippet>%(snippet)s</Snippet> -->
                <LookAt>
                  <longitude>%(fromlon).14f</longitude>
                  <latitude>%(fromlat).14f</latitude>
                  <range>50000</range>
                </LookAt>
                <MultiGeometry>
            """ % family )

            for line in family['inlawlines']:
                self.write_line(line)

            self.g.write("""
                </MultiGeometry>
            </Placemark>
            """)

    def write_line(self, line):
        self.g.write("""
            <LineString>
              <coordinates>%(fromlon).14f,%(fromlat).14f %(tolon).14f,%(tolat).14f</coordinates>
              <tessellate>1</tessellate>
            </LineString>
        """ % line )

    def write_placemark(self,placename, lat, lon, event):
        # the long description: 
        # long name of people to who event is coupled
        # Place: Placename
        # Eventtype: ...... Role: ....
        # Description: ....
        # Date: ....
        description = event[0][0] + '<br />' \
            + _('<b>Place</b>: %(placename)s') %{'placename' : placename} \
            + '<br />' \
            + _('<b>Eventtype</b>: %(eventtype)s')%{'eventtype': event[1]} \
            + ' ' + _('<b>Role</b>: %(role)s')%{'role': event[3]} \
            + '<br />' \
            + _('<b>Description</b>: %(eventdescription)s') % \
                {'eventdescription' : event[2]} \
            + '<br />' \
            + _('<b>Date</b>: %(date)s')%{'date' :event[4]}
        #icon depending on the event, make name
        iconend = ''
        if event[6] == RelLib.EventType.MARRIAGE :
            if event[0][2] == 'Fam' :
                iconend = 'mar'
        elif event[6] == RelLib.EventType.BIRTH :
            if event[0][2] != 'Fam' :
                iconend = 'birth'
        elif event[6] == RelLib.EventType.DEATH :
            if event[0][2] != 'Fam' :
                iconend = 'death'
        elif event[6] == RelLib.EventType.BAPTISM :
            if event[0][2] != 'Fam' :
                iconend = 'bapt'
        elif event[6] == RelLib.EventType.BURIAL :
            if event[0][2] != 'Fam' :
                iconend = 'burial'
        #WATCH out: coordinates are longitude, latitude, height
        try:
            self.g.write("""
            <name>%(name)s</name>
            <visibility>1</visibility>
            <open>1</open>
            <Snippet>%(snip)s</Snippet>
            <description>%(descr)s</description>
            <styleUrl>%(style)s</styleUrl>
            <LookAt>
              <latitude>%(lat).14f</latitude>
              <longitude>%(lon).14f</longitude>
              <range>500</range>
              <tilt>8.3</tilt>
            </LookAt>
            <Point>
              <coordinates>%(lon).14f,%(lat).14f,%(height).14f</coordinates>
            </Point>
""" % { 'name'   : event[0][1], \
        'style'  :'#gramps' + event[0][2]+iconend, \
        'lat' : lat, 'lon' : lon, 'height' : 0, \
        'snip'  : _('Place')+': ' + placename + ';' + _('Event')+': ' + event[2], \
        'descr' : description } )
        except TypeError :
            self.g.write("""
            <name>%(name)s</name>
            <visibility>1</visibility>
            <open>1</open>
            <Snippet>%(snip)s</Snippet>
            <description>%(descr)s</description>
            <styleUrl>%(style)s</styleUrl>
            <LookAt>
              <latitude> </latitude>
              <longitude> </longitude>
              <range>500</range>
              <tilt>8.3</tilt>
            </LookAt>
""" % { 'name'   : event[0][1], \
        'style'  :'#gramps' + event[0][2]+iconend, 
        'snip'  : _('Place')+': ' + placename + ';' + _('Event')+': ' + event[2], \
        'descr' : description } )


#------------------------------------------------------------------------
#
# GNOME/gtk
#
#------------------------------------------------------------------------
import gtk

#-------------------------------------------------------------------------
#
# load GRAMPS libraries
#
#-------------------------------------------------------------------------

import const
from QuestionDialog import ErrorDialog
from PluginUtils import register_report
from ReportBase import CATEGORY_CODE, MODE_GUI, MODE_CLI, Report, ReportOptions
from ReportBase._ReportDialog import ReportDialog
from NameDisplay import displayer as _nd
from DateHandler import displayer as _dd
#import DateHandler
import PlaceUtils

#------------------------------------------------------------------------
#
# Report class
#
#------------------------------------------------------------------------
class DescendNet:

    def __init__(self,database,person,options_class):
        self.database = database
        self.start_person = person

        options = options_class.handler.options_dict


#------------------------------------------------------------------------
#
# Constant options items
#
#------------------------------------------------------------------------

class _options:
    # internal ID, english option name (for cli), localized option name (for gui)
    formats = (
        ("kml", "Google Earth KML", _("Google Earth KML"), "application/klm"),
    )

#check if googleearth is installed. 
#  --> problem, googleearth is normally installed locally, not in the PATH variable... TODO
if os.sys.platform == "win32":
    _gge_found = Utils.search_for("googleearth.exe")
else:
    _gge_found = Utils.search_for("googleearth")




#------------------------------------------------------------------------
#
# Options class 
#
#------------------------------------------------------------------------
class DescendNetOptions(NewReportOptions):
    def enable_options(self):
        self.enable_dict = {}
        self.widgets = [
#            FilterWidget(self, label = _("Filter"),
#                         name = "filter",
#                         filters = ["all filters"]),
        CheckWidget(self, label = _("Show with unknown Location"),
                         name = "unknown",
                         value = 0, 
                         help = "Show events with known location",
                         ),
        CheckWidget(self, label = _("Show with unknown Position"),
                         name = "unknownlatlon",
                         value = 0,
                         help = "Show events with known location",
                         ),
        CheckWidget(self, label = _("Show with unknown Date"),
                         name = "unknowndate",
                         value = 1,
                         help = "Show events with known location",
                         ),
       ]
        


#-------------------------------------------------------------------------
#
#    The Dialog class
#
#-------------------------------------------------------------------------

class DescendNetDialog(ReportDialog):
    def __init__(self,dbstate,uistate,person):
        self.database = dbstate.db
        self.person = person
        name = "map_view"
        translated_name = _("Map View (Google Earth export)")
        self.options_class = DescendNetOptions(name)
        self.category = CATEGORY_CODE
        ReportDialog.__init__(self,dbstate,uistate,person,self.options_class,
                              name,translated_name)
        response = self.window.run()
        if response == gtk.RESPONSE_OK:
            try:
                self.make_report()
            except (IOError,OSError),msg:
                ErrorDialog(str(msg))
        elif response == gtk.RESPONSE_DELETE_EVENT:
            return
        self.close()

    def make_doc_menu(self,active=None):
        """Build a one item menu of document types that are
        appropriate for this report."""
        self.format_menu = FormatComboBox()
        self.format_menu.set()

    def make_document(self):
        """Do Nothing.  This document will be created in the
        make_report routine."""
        pass
    
    def setup_style_frame(self):
        """The style frame is not used in this dialog."""
        pass

    def parse_style_frame(self):
        """The style frame is not used in this dialog."""
        pass

    def doc_type_changed(self, obj):
        """The doc type is of no importance in this dialog."""
        pass

    def setup_paper_frame(self):
        """No need for Paper Options in this dialog"""
        pass
    def parse_paper_frame(self):
        """No need for Paper Options in this dialog"""
        pass

    def make_report(self):
        """Create the object that will produce the GraphViz file."""
        DescendNetExtract(self.database,self.person,self.options_class)


#------------------------------------------------------------------------
#
# Combo Box classes
#
#------------------------------------------------------------------------
class FormatComboBox(gtk.ComboBox):
    """
    Format combo box class.
    
    Trivial class supporting only one format.
    """

    def set(self,tables=0,callback=None,obj=None,active=None):
        self.store = gtk.ListStore(str)
        self.set_model(self.store)
        cell = gtk.CellRendererText()
        self.pack_start(cell,True)
        self.add_attribute(cell,'text',0)
        self.store.append(row=["Google Earth (kml)"])
        self.set_active(0)

    def get_label(self):
        return "Google Earth (kml)"

    def get_reference(self):
        return None

    def get_paper(self):
        return 1

    def get_styles(self):
        return 0

    def get_ext(self):
        return '.kml'

    def get_printable(self):
        _apptype = _options.formats[self.get_active()][3]
        print_label = None
        try:
            mprog = Mime.get_application(_apptype)
    
            if Utils.search_for(mprog[0]):
                print_label = _("Open in %(program_name)s") % { 'program_name':
                                                        mprog[1]}
            else:
                print_label = None
        except:
            print_label = None

        return print_label

    def get_clname(self):
        return 'kml'

class GraphicsFormatComboBox(gtk.ComboBox):
    """
    Format combo box class for graphical (not codegen) report.
    """

    def set(self,active=None):
        self.store = gtk.ListStore(str)
        self.set_model(self.store)
        cell = gtk.CellRendererText()
        self.pack_start(cell,True)
        self.add_attribute(cell,'text',0)
        active_index = 0
        index = 0
        for item in _options.formats:
            self.store.append(row=[item[2]])
            if active == item[0]:
                active_index = index
            index = index + 1
        self.set_active(active_index)

    def get_label(self):
        return _options.formats[self.get_active()][2]

    def get_reference(self):
        return EmptyDoc

    def get_paper(self):
        return 0

    def get_styles(self):
        return 0

    def get_ext(self):
        return '.%s' % _options.formats[self.get_active()][0]

    def get_format_str(self):
        return _options.formats[self.get_active()][0]

    def get_printable(self):
        _apptype = _options.formats[self.get_active()][3]
        print_label = None
        try:
            mprog = Mime.get_application(_apptype)
            if Utils.search_for(mprog[0]):
                print_label = _("Open in %(program_name)s") % { 'program_name':
                                                                mprog[1] }
            else:
                print_label = None
        except:
            print_label = None
        return print_label

    def get_clname(self):
        return 'print'

#------------------------------------------------------------------------
#
# Empty class to keep the BaseDoc-targeted format happy
#
#------------------------------------------------------------------------
class EmptyDoc:
    def __init__(self,styles,type,template,orientation,source=None):
        self.print_req = 0

    def init(self):
        pass
    
    def print_requested(self):
        self.print_req = 1


#------------------------------------------------------------------------
#
#  Class that extracts the geographical data corresponding to the options
#    and sends that to the backend
#
#------------------------------------------------------------------------

class DescendNetExtract:
  
    def __init__(self,database,person,options_class):
        """
        Creates DescendNetExtract object that produces data for the report.
        
        The arguments are:

        database        - the GRAMPS database instance
        person          - currently selected person
        options_class   - instance of the Options class for this report

        This report needs the following parameters (class variables)
        that come in the options class.
        
           filter  : from who to export data: current person, his family or 
                        a custom filter
           events  : which events to show
           unknowns: if place has unknown location/event has unknown location, 
                       export it nevertheless
                       
        """
        self.database = database
        self.start_person = person
        
        options = options_class.handler.options_dict
        # selection of who to extract events from 
        
        # selection of which events to show
        self.onlyKnownPlace = not options['unknown']
        self.onlyLatLonPlace = not options['unknownlatlon']
        self.onlyDatePlace = not options['unknowndate']
        self.options_class = options_class
        
        self.filename = options_class.get_output()
        
        self.obtain_data()
        self.writetobackend()

    def obtain_data(self):
        """we perform the needed lookup in the database. 
          output needed, a dictionary of placemarks, with index the placehandle
            and a placemark being 
        """
        self.placelines = {}
        
        # if person, obtain the events, and place them in the placemarks if needed
        self.title = _('Descendants' )     

        # start recursing at the main person
        self.recurse_data(self.start_person, 1)

    def recurse_data(self, person, gen):
        
        for family_handle in person.get_family_handle_list():
            family = self.database.get_family_from_handle(family_handle)
            targets = set()
            childs = []
            inlaws = []
            for childref in family.get_child_ref_list():
                child_handle = childref.get_reference_handle()
                child = self.database.get_person_from_handle(child_handle)

                to_event_ref = child.get_birth_ref()
                if to_event_ref:
                    targets.add(to_event_ref)
                    childs.append(child)
                    
                    self.recurse_data(child, gen+1)


            mother_handle = family.get_mother_handle()
            mother = self.database.get_person_from_handle(mother_handle)
            if mother:
                inlaw = {}
                mother_event_ref = mother.get_birth_ref()
                if mother_event_ref:
                    inlaw['partner'] = mother_event_ref
                
                mother_family_handle = mother.get_main_parents_family_handle()
                mother_family = self.database.get_family_from_handle(mother_family_handle)
                if mother_family:
                
                    mother_mother_handle = mother_family.get_mother_handle()
                    mother_mother = self.database.get_person_from_handle(mother_mother_handle)
                    if mother_mother:
                        mother_mother_event_ref = mother_mother.get_birth_ref()
                        if mother_mother_event_ref:
                            inlaw['partnermother'] = mother_mother_event_ref

                    mother_father_handle = mother_family.get_father_handle()
                    mother_father = self.database.get_person_from_handle(mother_father_handle)
                    if mother_father:
                        mother_father_event_ref = mother_father.get_birth_ref()
                        if mother_father_event_ref:
                            inlaw['partnerfather'] = mother_father_event_ref
            
                inlaws.append( inlaw )

            self.add_family(gen, person, targets, childs, inlaws)


    def eventref_to_latlon(self, event_ref):
        if not event_ref:
            return (None,None)
        event = self.database.get_event_from_handle(event_ref.get_reference_handle())
        if not event:
            return (None,None)
        place_ref = event.get_place_handle()
        if not place_ref:
            return (None,None)
        place = self.database.get_place_from_handle(place_ref)
        if not place:
            return (None,None)
        lat   = place.get_latitude()
        lon   = place.get_longitude()
        if not lat or not lon:
            return (None, None)
        return PlaceUtils.conv_lat_lon(lat,lon,"D.D8")


    def add_family(self, gen, father, targets, childs, inlaws):
        from_event_ref = father.get_birth_ref()
        lat1, lon1 = self.eventref_to_latlon(from_event_ref)
        if not lat1 or not lon1:
            return

        father_name = _nd.display_name(father.get_primary_name())
        name = father_name
        snippet = '%d Kinder' % len(childs)

        description = 'Kinder:\n'
        for child in childs:
            description += '%s\n' % _nd.display_name(child.get_primary_name())

        lines = []
        for event2 in targets:
            if event2:
                lat2, lon2 = self.eventref_to_latlon(event2)
                if lat2 and lon2 and lat1 != lat2 and lon1 != lon2:
                    lines.append({
                        'fromlat' : float(lat1),
                        'fromlon' : float(lon1),
                        'tolat'   : float(lat2),
                        'tolon'   : float(lon2),
                        })
        inlawlines = []
        for inlaw in inlaws:
            if 'partner' in inlaw:
                lat2, lon2 =  self.eventref_to_latlon(inlaw['partner'])
                if lat2 and lon2 and lat1 != lat2 and lon1 != lon2:
                    inlawlines.append({
                        'fromlat' : float(lat2),
                        'fromlon' : float(lon2),
                        'tolat'   : float(lat1),
                        'tolon'   : float(lon1),
                        })
                    if 'partnermother' in inlaw:
                        lat3, lon3 =  self.eventref_to_latlon(inlaw['partnermother'])
                        if lat3 and lon3 and lat3 != lat2 and lon3 != lon2:
                            inlawlines.append({
                                'fromlat' : float(lat3),
                                'fromlon' : float(lon3),
                                'tolat'   : float(lat2),
                                'tolon'   : float(lon2),
                                })
                    if 'partnerfather' in inlaw:
                        lat3, lon3 =  self.eventref_to_latlon(inlaw['partnerfather'])
                        if lat3 and lon3 and lat3 != lat2 and lon3 != lon2:
                            inlawlines.append({
                                'fromlat' : float(lat3),
                                'fromlon' : float(lon3),
                                'tolat'   : float(lat2),
                                'tolon'   : float(lon2),
                                })
        
        if gen not in self.placelines:
            self.placelines[gen] = []
        self.placelines[gen].append( {
            'gen'     : gen,
            'name'    : name,
            'description': description,
            'snippet' : snippet,
            'fromlat' : float(lat1),
            'fromlon' : float(lon1),
            'lines'   : lines,
            'inlawlines' : inlawlines,
            }
        )
                        
    def writetobackend(self):
         """ write to the actual backend by calling it's writer correctly
         """
         kmlfile = KmlWriter(self.title, self.placelines)
         kmlfile.write(self.filename)


#------------------------------------------------------------------------
# 
#    Report registration
#
#------------------------------------------------------------------------

def cl_report(database,name,category,options_str_dict):
    clr = CommandLineReport(database,name,category,DescendNetOptions,
                            options_str_dict)
    # Exit here if show option was given
    if clr.show:
        return

    report = DescendNetExtract(database,clr.person,clr.option_class)

register_report(
    name = 'kmldescexport',
    category = CATEGORY_CODE,
    report_class = DescendNetDialog,
    #options_class = DescendNetOptions,
    options_class = cl_report,
    modes = MODE_GUI | MODE_CLI,
    translated_name = _("KML Descendand Export"),
    status = _("UnStable"),
    description = _("Creates a kml file that can be played/opened in Google Earth."),
    author_name = "Joachim Breitner",
    author_email = "mail@joachim-breitner.de"
    )
