{-# LANGUAGE GeneralizedNewtypeDeriving #-}

import Data.List
import Data.Ord
import Graphics.Rendering.Cairo
import Data.Tree
import Data.Maybe
import Control.Monad
import Control.Monad.Writer
import Control.Monad.Reader
import System
import Debug.Trace

import qualified Data.Set as S
import qualified Data.Map as M

-- Settings
width, height, pad, pagepad, lineheight, yearheight:: Num a => a
width = 2000
height = 768
pad = 10
pagepad = 10
lineheight = 100
yearheight = 3

this_year = 2008

-- Child placer selector
child_placer :: [Contour] -> [Double]
child_placer = {- child_placer_greedy -}  gen_placer

stupid_child_placer :: [Contour] -> [Double]
stupid_child_placer = gen_placer_greedy

-- Generic place for any kind of objects a with a given distance function
gen_placer :: (Metric a) => [a] -> [Double]
gen_placer = {- gen_placer_avg -} gen_placer_even {- gen_placer_greedy -}

centerer :: Int -> [Contour] -> (Contour, Double)
centerer = centerer_flat
-- centerer = centerer_deep

-- Code...

-- Building a Tree structure from a list of ids
tree :: [(String, a)] -> Tree (a)
tree ids = unfoldTree (\(i,d) -> (d, filter (\(oi,_) -> i `isPrefixOf` oi && length oi == length i + 1 ) ids )) first
  where (Just first) = find ((=="1").fst) ids

-- Type for the contour and edge of a subtree (relative to the center = root) to the left and to the right
data Contour = Contour Int [(Double, Double)]
type Edge    = [Double]
type Color = (Double, Double, Double)

data NodeInfo = NodeInfo {
			edgeColor :: Color,
			fromY :: Maybe Int,
			toY :: Maybe Int,
			gen :: Int,
			num :: String
			}

class Metric a where
	dist :: a -> a -> Double

instance Metric Contour where
	dist = cdistance

-- Render-like Monad to record edge coordinates and positions

type MyRender a = Writer [(Double,Double,Double,Double,Color)] a

myTranslate :: Double -> Double -> MyRender a -> MyRender a
myTranslate dx dy = censor $ map $ \(x1,y1,x2,y2,(r,g,b))
                           -> (x1 + dx, y1 + dy, x2 + dx, y2 + dy, (r,g,b))

myEdge :: (Double,Double,Double,Double,Color) -> MyRender ()
myEdge e = tell [e]

renderMyRender :: ((Double, Double) -> (Double, Double)) -> MyRender a -> Render a
renderMyRender placement rend = do
    let (ret, edges) = runWriter rend
    forM edges $ \(x1,y1,x2,y2,c) -> do
        niceLine (placement (x1,y1)) (placement (x2,y2)) c 
    return ret

normalLine (x1,y1) (x2,y2) (r,g,b) = do
    moveTo x1 y1
    setSourceRGB r g b
    lineTo x2 y2
    stroke

niceLine (x1,y1) (x2,y2) (r,g,b) = do
    setSourceRGB r g b
    moveTo x1 y1
    arcNegative x2 y2 radius (angle-angled) (angle+angled)
    closePath
    fill
  where
    angle = pi/2 - atan2 (x1-x2) (y1-y2) 
    angled = pi/2 - sin (radius/len)
    len = sqrt((x1-x2)^2 + (y1-y2)^2)
    radius = 1.5

-- Placement methods

rectangular (x,y) = (x,y)

mirrored (x,y) = (-x,y)

onepi c w (x,y) = (y * cos alpha, y * sin alpha)
  where alpha = pi - ((x+c)/w) * pi

onepi_spaced c w (x,y) = (y * cos alpha, y * sin alpha)
  where alpha' = pi - ((x+c)/w) * pi
        alpha | alpha' <  s = alpha' * ((pi/2-pad)/s)
              | alpha' >= s = pi - (pi - alpha') * ((pi/2-pad)/(pi-s))
        pad = 0 * pi/180
        s = (90 + 3.3) * pi/180

morepi c w (x,y) = (y * cos alpha, y * sin alpha)
  where alpha = (pi + extra) - ((x+c)/w) * (pi + 2* extra + extra2)
        extra = asin (7.0/17.0)
        extra2 = 45* pi/180


-- Helper functions concerning contours
cempty gen = Contour gen []

cgen (Contour g _ ) = g

left,right :: Contour -> Edge
left (Contour _ c) = map fst c
right (Contour _ c) = map snd c

ccenter, cwidth :: Contour -> Double
cwidth (Contour _ []) = 0
cwidth c = maximum (left c) + maximum (right c)

ccenter (Contour _ []) = 0
ccenter c = maximum (left c)

clength :: Num n => Contour -> n
clength (Contour _ c) = fromIntegral (length c)

ctip :: Contour -> Contour
ctip (Contour g [])    = Contour g []
ctip (Contour g (t:_)) = Contour g [t]

cmove :: Double -> Contour -> Contour
cmove d (Contour g c) = Contour g $ map (\(l,r) -> (l-d, r+d)) c

cenlarge n h (Contour g c) = Contour g (replicate n h ++ c)
creplicate g n h = Contour g (replicate n h)
surround (Contour _ []) = (0,0)
surround c = (ccenter c + pad, cwidth c - ccenter c + pad)
padLeft (Contour _ []) = (0,0)
padLeft c = (0, cwidth c - ccenter c+ pad)

-- Experimental generic child spacers

-- Distance function for subtree contours
cdistance (Contour g []) _ = pad/fromIntegral g
cdistance _ (Contour g []) = pad/fromIntegral g
cdistance c1 c2 = pad/fromIntegral (cgen c1-1) + (maximum (zipWith (+) (right c1) (left c2)))


-- Simple greedy algorithm. Gives same results as the child_placer_greedy,
-- just slower
gen_placer_greedy []     = []
gen_placer_greedy [x]    = [0]
gen_placer_greedy (x:xs) = minimum (zipWith (\pos y -> pos - dist x y) others xs) : others
  where	others = gen_placer_greedy xs

-- Algorithm that tries to evenly add space between all, then all but the
-- last, and so on element
gen_placer_even l = map snd $ gen_placer_even' l

gen_placer_even' []     = []
gen_placer_even' [x]    = [(x,0)]
gen_placer_even' (x:xs) = new : moved ++ unmoved
  where	others = gen_placer_even' xs
        (moveable, unmoved, pos) = find_hit x others
	new = (x,pos)
	moved = iter_inits (spread new) moveable

-- Helper function to do the spread as often as possible. Could be
-- optimized, I guess, by checking which element caused the spread to stop
iter_inits :: ([a] -> [a]) -> [a] -> [a]
iter_inits f [] = []
iter_inits f l  = iter_inits f (init this) ++ [last this]
  where this = f l

-- Spread all but the last element of movable evenly to the left
spread new [a] = [a]
spread new moveable = moved
  where	slots = length moveable 
	minspace = minimum $ zipWith freespace (init moveable) (tail moveable)
	-- allowed distances minus the already minimal space between two elements:
	distances = map (subtract minspace . freespace new) (init moveable) 
  	slot_move = minimum $ zipWith (\d n -> d / (fromIntegral (slots-n))) distances [0..slots]
	moved = zipWith (\(x,pos) n -> (x, pos - (fromIntegral (slots-n))*slot_move)) moveable [1..slots]

freespace :: Metric a => (a, Double) -> (a, Double) -> Double
freespace (x,p1) (y,p2) = p2 - p1 - dist x y

find_hit a []     = ([],[],0)
find_hit a ((x,npos):xs) = if tpos <= mpos then ([(x,npos)],  moved ++ unmoved, tpos)
                                           else ( (x,npos):moved,      unmoved, mpos)
  where	tpos = npos - dist a x
        (moved, unmoved, mpos) = find_hit a xs

-- Averaging algorithm, unfortunately wrong
gen_placer_avg l = zipWith (\a b -> (a + b)/2) lean_left lean_right
  where lean_left = align $ map negate $ reverse (gen_placer_greedy (reverse l))
        lean_right = align $ gen_placer_greedy l
	align []     = []
	align (x:xs) = map (subtract x) (x:xs)

stupid_gen_placer :: (Metric a) => [a] -> [Double]
stupid_gen_placer [] = []
stupid_gen_placer [_] = [0]
stupid_gen_placer (x:xs) = 0 : map (+d) others
  where d = pad + maximum (map (dist x) xs)
        others = stupid_gen_placer xs

-- Old direct spacer algorithm

-- Fits a subtree close to the predecessor, passing on the new temporary edge, and returns the
-- subtree’s new position
child_placer_greedy = snd . mapAccumL fit_child [0]

fit_child :: Edge -> Contour -> (Edge, Double)
fit_child l cont =  (r, cx)
  where	cx = maximum (zipWith (+) l (left cont))
  	moved_cont = cmove cx cont
  	r  = overlay_list l (map (+pad) (right moved_cont))

-- Given a list and an overlay, overlay the list with the values of the overlay
overlay_list :: [t] -> [t] -> [t]
overlay_list l      []     = l
overlay_list []     r      = r
overlay_list (_:ls) (r:rs) = r : overlay_list ls rs

-- Given a list and an overlay, overlay the list with the values of the overlay
overlay_contours (Contour g c1) (Contour _ c2) = Contour g (overlay_list c1 c2)
overlay_contours_left c1 c2 = Contour (cgen c1) $ zip (overlay_list (left c1) (left c2)) (right c1)

-- Given subtree contours build the new contour
merge_contours :: Int -> [Contour] -> Contour
merge_contours g cs = Contour g $ zip l r
  where	l = foldl overlay_list [] (reverse (map left cs))
        r = foldl overlay_list [] (map right cs)

centerer_deep :: Int -> [Contour] -> (Contour, Double)
centerer_deep g contours = (contour, center)
  where off_center_contour = merge_contours g contours
	center = (cwidth off_center_contour)/2 - ccenter off_center_contour
	contour = cmove (-center) off_center_contour

centerer_flat :: Int -> [Contour] -> (Contour, Double)
centerer_flat g contours = (contour, center)
  where off_center_contour = merge_contours g contours
	center = (cwidth1 off_center_contour)/2 - ccenter1 off_center_contour
	contour = cmove (-center) off_center_contour
	cwidth1  = cwidth . ctip 
	ccenter1 = ccenter . ctip 

centerer_left :: Int -> [Contour] -> (Contour, Double)
centerer_left g contours = (contour, center)
  where off_center_contour = merge_contours g contours
	center = -pagepad - ccenter off_center_contour
	contour = cmove (-center) off_center_contour

-- Renders a subtree and returns it’s contour
render :: Tree NodeInfo -> (Contour, (MyRender (), NodeInfo))
render (Node ni c) = (enlarged, (r, ni))
  where (child_contours, child_renderers) = unzip $ map render c
	child_places = child_placer child_contours
	subcontours = zipWith cmove child_places child_contours
	(contour, center) = centerer (gen ni) subcontours
	enlarged = cenlarge 1 (0,0) contour
	r = do 
		zipWithM_ draw_child child_places child_renderers
		zipWithM_ draw_self  child_places child_renderers

	draw_child x (r, cni) = do
		myTranslate (x - center) lineheight $ r

	draw_self x (r, cni) = do
                myEdge (0,0,x-center,lineheight,edgeColor cni)

child_offset from (NodeInfo {fromY = Nothing}) = 0
child_offset from (NodeInfo {fromY = Just f})  = fromIntegral $ yearheight * (f - from)

-- Renders a subtree and returns it’s contour
render2 :: Tree NodeInfo -> (Contour, (Render (), NodeInfo))
render2 (Node ni c) | isNothing (fromY ni) = (cempty (gen ni), (return (), ni))
		    | otherwise = (enlarged, (r, ni))
  where children = filter (isJust . fromY . rootLabel) c
  	(child_contours, child_renderers) = unzip $ map render2 children
	enlarged_child_contours = zipWith enlargeChildCont children child_contours
	  where enlargeChildCont cn cont = cenlarge (child_offset from (rootLabel cn)) (0,0) cont
	child_places = stupid_child_placer enlarged_child_contours
	subcontours = zipWith cmove child_places enlarged_child_contours
	(children_contour, children_center) = centerer_flat (gen ni) subcontours
	own_contour = creplicate (gen ni) (yearheight *(to-from+10)) $ surround children_contour
	enlarged = overlay_contours children_contour own_contour
	width = cwidth own_contour
	center = ccenter own_contour
	from = fromJust $ fromY ni
	to   = fromMaybe this_year $ toY ni
	height = fromIntegral $ yearheight * (to-from+1)
	draw_child x (r, cni) = do
		save
		translate (x - children_center) (child_offset from cni)
		r
		restore
	draw_self = do
		save
		rectangle (-center) 0 width height
		let g = 1 - (fromIntegral (gen ni))/12 in setSourceRGB g g g
		fill
		let (r,g,b) = edgeColor ni in setSourceRGB r g b
		moveTo (-center) height
		lineTo (-center) 0
		lineTo (width-center) 0
		lineTo (width-center) height
		when (to /= this_year) $ closePath
		stroke
		restore
	r = do  draw_self
		zipWithM_ draw_child child_places child_renderers

render3 :: Tree NodeInfo -> (Contour, (Render (), NodeInfo))
render3 (Node ni c) | isNothing (fromY ni) = (cempty (gen ni), (return (), ni))
		    | otherwise = (enlarged, (r, ni))
  where children = filter (isJust . fromY . rootLabel) c
  	(child_contours, child_renderers) = unzip $ map render3 children
	enlarged_child_contours = zipWith enlargeChildCont children child_contours
	  where enlargeChildCont cn cont = cenlarge (child_offset from (rootLabel cn)) (0,0) cont
	child_places = child_placer enlarged_child_contours
	subcontours = zipWith cmove child_places enlarged_child_contours
	(children_contour, children_center) = centerer_left (gen ni) subcontours
	own_contour = creplicate (gen ni) (yearheight *(to-from+1)) $ padLeft children_contour
	enlarged = overlay_contours_left children_contour own_contour
	center = ccenter own_contour
	from = fromJust $ fromY ni
	to   = fromMaybe this_year $ toY ni
	height = fromIntegral $ yearheight * (to-from+1)
	draw_child x (r, cni) = do
		save
		setLineWidth 1
		setSourceRGBA 0.5 0.5 0.5 0.5
		moveTo (-center) (child_offset from cni)
		lineTo (x - children_center) (child_offset from cni)
		stroke
		restore
		save
		translate (x - children_center) (child_offset from cni)
		r
		restore
	draw_self = do
		save
		let (r,g,b) = edgeColor ni in setSourceRGB r g b
		moveTo (-center) 0
		lineTo (-center) height
		stroke
		restore
	r = do  draw_self
		zipWithM_ draw_child child_places child_renderers

fitOn r c w h = do
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	setLineCap LineCapRound
	scale (width / (w + 2*pagepad)) (height / h)
	translate (c + pagepad) pagepad
	r

fitOnWidth r c w = do
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	setLineCap LineCapRound
	scale (width / (w + 2*pagepad)) 1
	translate (c + pagepad) pagepad
	r

unscaled r c  = do
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	setLineCap LineCapRound
	setLineWidth 2
	translate (c + pagepad) pagepad
	r

leftCenter h r = do
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	setLineCap LineCapRound
        setLineWidth 0.7
        translate pagepad (h)
        r
    
topCenter w r = do
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	setLineCap LineCapRound
        setLineWidth 0.7
        translate (w+pagepad) pagepad
        r

fromTop w r = do
	setSourceRGB 1 1 1
	paint
	setSourceRGB 0 0 0
	setLineCap LineCapRound
        setLineWidth 0.7
        translate (w+pagepad) (pagepad+w/17*7)
        arc 0 0 (lineheight/12) 0 (2*pi)
        fill
        r
    

parse_line l = (i, NodeInfo { edgeColor = color, fromY = from, toY = to, gen = gen, num = num  })
  where [i,d,g,t] = words l
        {-
        color | d == "TOT"   = (0,0,0)
              | d == "OK"    = (0,1,0)
              | d == "MINDJ" = (0,0,1)
              | d == "TODO"  = (1,0,0)
              | d == "NOTOK"  = (1,0.5,0.5)
        -}
        color = (0,0,0)
	fromPy s | s == "None" = Nothing
	         | s == "0"    = Nothing
	         | otherwise   = Just (read s)
	from = fromPy g
	to = fromPy t
	gen = length i
	num = 'I':i


writers :: String -> Int -> Int -> Render a -> IO ()
writers basename width height r = do
	withImageSurface FormatRGB24 width height $ \sur -> do
		renderWith sur r
		surfaceWriteToPNG sur (basename ++ ".png")
	withSVGSurface (basename ++ ".svg") (fromIntegral width) (fromIntegral height) $ \sur -> do
		renderWith sur r
	withPDFSurface (basename ++ ".pdf") (fromIntegral width) (fromIntegral height) $ \sur -> do
		renderWith sur r
	return ()


dot_input tree = 
	"digraph ft { {"++
	"node [shape=plaintext, fontsize=16]"++
	concat (intersperse "->" (map show decades)) ++
	"}  { node [style=invis] " ++
	concatMap (\id -> id ++ "b [group="++id++"];\n") ids ++
	concatMap (\id -> id ++ "d [group="++id++"];\n") ids ++
	"}" ++
	concatMap (\(d,s) -> "{rank=same; "++show d++";"++concatMap (++";") (S.toList s)++"}\n") (M.toList events) ++
	concatMap (\e -> num e ++ "b -> " ++ num e ++ "d[dir=none,style=\"setlinewidth(9)\"];\n") elems++
	rels tree++
	"}"
  where elems = filter (isJust . fromY) $ flatten tree
  	births = M.fromListWith (S.union) $ map (\e -> ((dec.fromJust.fromY) e, S.singleton(num e++"b"))) $ elems
  	deaths = M.fromListWith (S.union) $ map (\e -> ((dec.fromMaybe 2008.toY) e, S.singleton(num e++"d"))) $ elems
	events = M.unionWith (S.union) births deaths
	decades = nub $ sort $ M.keys births ++ M.keys deaths
  	ids = map num elems
	dec = (*10).(`div`10)
	rels (Node ni cs) = concatMap (\c -> num ni ++"b -> " ++ num (rootLabel c) ++"b;\n") cs ++
			    concatMap rels cs


-- Load the data, turn it to a tree, calculate the contour and place it nicely fullimage
main = do
	[inFile] <- getArgs
	ids <- (map parse_line . lines) `fmap` readFile inFile
	let t = tree ids
	let (c1, (r1,_)) = render t
	--let scaled1 = fitOn r1 (ccenter c1) (cwidth c1) (lineheight * clength c1)
	let scaled1 = unscaled (renderMyRender rectangular r1) (ccenter c1)

        let radius = lineheight * (clength c1 - 1)
	let radial1 = fromTop radius $
                      renderMyRender (morepi (ccenter c1) (cwidth c1)) r1

	let (c2, (r2,_)) = render2 t
	--let scaled2 = fitOnWidth r2 (ccenter c2) (cwidth c2)
	let scaled2 = unscaled r2 (ccenter c2)

	let (c3, (r3,_)) = render3 t
	--let scaled3 = fitOnWidth r3 (ccenter c3) (cwidth c3)
	let scaled3 = unscaled r3 (ccenter c3)

	--writers "/tmp/graph" width height scaled1
    	--writers "/tmp/graph"  (round (cwidth c1)) (lineheight * clength c1 + 2*pagepad) scaled1
        writers "/tmp/graph" (round (radius*2) + 2*pagepad) (round (radius * 24/17) + 2*pagepad) radial1

	--writers "/tmp/graphR" (round (cwidth c2)) (clength c2 + 2*pagepad) scaled2
	
	--writers "/tmp/graphG" (round (cwidth c3)) (clength c3 + 2*pagepad) scaled3

	writeFile "/tmp/graph.dot" (dot_input t)
